
import java.awt.*;
import java.awt.image.BufferedImage;
import java.util.Random;

public abstract class AbstractMap {
    private IDrawingObject _drawingObject = null;
    protected int[][] _map = null;
    protected int _width;
    protected int _height;
    protected float _size_x;
    protected float _size_y;
    protected final Random _random = new Random();
    protected final int _freeRoad = 0;
    protected final int _barrier = 1;

    public BufferedImage CreateMap(int width, int height, IDrawingObject drawningObject)
    {
        _width = width;
        _height = height;
        _drawingObject = drawningObject;
        GenerateMap();
        while (!SetObjectOnMap())
        {
            GenerateMap();
        }
        return DrawMapWithObject();
    }

    public BufferedImage MoveObject(Direction direction)
    {
        if(_drawingObject!=null){
            float[] cortege = _drawingObject.GetCurrentPosition();
            if (Check(cortege[0],cortege[1],cortege[2],cortege[3]) != 0)
            {
                _drawingObject.MoveObject(GetOpositDirection(direction));
            }
            if(true){
                _drawingObject.MoveObject(direction);
            }
            return  DrawMapWithObject();
        }
        return null;
    }

    private Direction GetOpositDirection(Direction dir)
    {
        switch (dir)
        {
            case None:
                return Direction.None;
            case Up:
                return Direction.Down;
            case Down:
                return Direction.Up;
            case Left:
                return Direction.Right;
            case Right:
                return Direction.Left;
        }
        return Direction.None;
    }

    private boolean SetObjectOnMap()
    {
        if (_drawingObject == null || _map == null)
        {
            return false;
        }
        int x = _random.nextInt(0, 10);
        int y = _random.nextInt(0, 10);
        _drawingObject.SetObject(x, y, _width, _height);
        float[] cortege = _drawingObject.GetCurrentPosition();

        while (Check(cortege[0], cortege[1], cortege[2], cortege[3]) != 2)
        {
            int result;
            do
            {
                result = Check(cortege[0], cortege[1], cortege[2], cortege[3]);
                if (result == 0)
                {
                    _drawingObject.SetObject((int)cortege[0], (int)cortege[1], _width, _height);
                    return true;
                }
                else
                {
                    cortege[0] += _size_x;
                }
            } while (result != 2);
            cortege[0] = x;
            cortege[1] += _size_y;
        }
        return false;
    }

    private int Check(float Left, float Right, float Top, float Bottom)
    {
        int startX = (int)(Left / _size_x);
        int startY = (int)(Right / _size_y);
        int endX = (int)(Top / _size_x);
        int endY = (int)(Bottom / _size_y);
        if (startX < 0 || startY < 0 || endX >= _map[0].length || endY >= _map.length)
        {
            return 2;
        }
        for (int i = startX; i <= endX; i++)
        {
            for (int j = startY; j <= endY; j++)
            {
                if (_map[i][j] == _barrier)
                {
                    return 1;
                }
            }
        }
        return 0;
    }

    private BufferedImage DrawMapWithObject()
    {
        BufferedImage bmp = new BufferedImage(_width,_height,BufferedImage.TYPE_INT_RGB);
        if (_drawingObject == null || _map == null)
        {
            return bmp;
        }
        Graphics gr = bmp.getGraphics();
        for (int i = 0; i < _map.length; ++i)
        {
            for (int j = 0; j < _map.length; ++j)
            {
                if (_map[i][j] == _freeRoad)
                {
                    DrawRoadPart(gr, i, j);
                }
                    else if (_map[i][j] == _barrier)
                {
                    DrawBarrierPart(gr, i, j);
                }
            }
        }
        _drawingObject.DrawningObject(gr);
        return bmp;
    }

    protected abstract void GenerateMap();
    protected abstract void DrawRoadPart(Graphics g, int i, int j);
    protected abstract void DrawBarrierPart(Graphics g, int i, int j);
}

