import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.image.BufferedImage;
import java.util.Random;

public class FormWarship extends JFrame {
    private DrawingWarship _warship;
    private JPanel mainPanel;
    private JPanel drawPanel;
    private JButton buttonCreate;
    private JButton buttonLeft;
    private JButton buttonUp;
    private JButton buttonDown;
    private JButton buttonRight;
    private JToolBar toolBar;
    private JLabel toolBarLabelSpeed;
    private JLabel toolBarLabelWieght;
    private JLabel toolBarLabelColor;
    private JButton buttonCreateModif;

    public FormWarship(){
        InitializeComponent();
    }

    private void Draw(){
        Graphics2D graphics = (Graphics2D) drawPanel.getGraphics();
        graphics.clearRect(0, 0, drawPanel.getWidth(), drawPanel.getHeight());
        drawPanel.paintComponents(graphics);
        _warship.DrawTransport(graphics);
    }

    private void SetData(){
        Random rnd = new Random();
        _warship.SetPosition(rnd.nextInt(10, 100), rnd.nextInt(10, 100), mainPanel.getWidth(), mainPanel.getHeight());
        toolBarLabelSpeed.setText("Color: " + _warship.GetWarship().GetSpeed() + "  ");
        toolBarLabelWieght.setText("Weight: " + _warship.GetWarship().GetWeight() + "  ");
        toolBarLabelColor.setText("Color: " + _warship.GetWarship().GetBodyColor().getRed() + " " +
                _warship.GetWarship().GetBodyColor().getGreen() + " " + _warship.GetWarship().GetBodyColor().getBlue());
    }

    private void resizeWindow() {
        _warship.ChangeBorders(drawPanel.getWidth(), drawPanel.getHeight());
        Draw();
    }

    private void InitializeComponent() {
        setContentPane(mainPanel);
        setTitle("Warship");
        setSize(900, 700);
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
        setVisible(true);

        Icon iconUp = new ImageIcon("src\\Images\\ArrowUp.jpg");
        buttonUp.setIcon(iconUp);
        Icon iconDown = new ImageIcon("src\\Images\\ArrowDown.jpg");
        buttonDown.setIcon(iconDown);
        Icon iconLeft = new ImageIcon("src\\Images\\ArrowLeft.jpg");
        buttonLeft.setIcon(iconLeft);
        Icon iconRight = new ImageIcon("src\\Images\\ArrowRight.jpg");
        buttonRight.setIcon(iconRight);

        //кнопка добавления объекта
        buttonCreate.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Random rnd = new Random();
                _warship = new DrawingWarship(rnd.nextInt(100, 300), rnd.nextInt(1000, 2000),
                        new Color(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)), rnd.nextInt(3));
                SetData();
                Draw();
            }
        });

        //добавление модифицированного объекта
        buttonCreateModif.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                Random rnd = new Random();
                _warship = new DrawingAircraftCarrier(rnd.nextInt(100, 300), rnd.nextInt(1000, 2000),
                        new Color(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                        new Color(rnd.nextInt(256), rnd.nextInt(256), rnd.nextInt(256)),
                        rnd.nextBoolean(), rnd.nextBoolean(), rnd.nextBoolean());
                SetData();
                Draw();
            }
        });

        buttonLeft.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_warship != null) _warship.MoveTransport(Direction.Left);
                Draw();
            }
        });

        buttonRight.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_warship != null) _warship.MoveTransport(Direction.Right);
                Draw();
            }
        });

        buttonUp.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_warship != null) _warship.MoveTransport(Direction.Up);
                Draw();
            }
        });

        buttonDown.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                if (_warship != null) _warship.MoveTransport(Direction.Down);
                Draw();
            }
        });

        mainPanel.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(ComponentEvent e) {
                super.componentResized(e);
                resizeWindow();
            }
        });
    }
}
